@@ -5,12 +5,13 @@ from collections.abc import Callable, Iterable, Mapping
 from copy import deepcopy
 from json import dumps
 from logging import DEBUG, getLogger
-from pathlib import Path
-from typing import TYPE_CHECKING, Any, Generic, Literal, TypeAlias, TypedDict, TypeVar
+from typing import TYPE_CHECKING, Any, ClassVar, Generic, Literal, TypeAlias, TypeVar
 
 from requests import Response, get
+from typing_extensions import TypedDict
 
-from wg_utilities.clients.oauth_client import OAuthClient, StrBytIntFlt
+from wg_utilities.clients.json_api_client import StrBytIntFlt
+from wg_utilities.clients.oauth_client import OAuthClient
 from wg_utilities.loggers import add_stream_handler
 
 LOGGER = getLogger(__name__)
@@ -31,13 +32,13 @@ GetJsonResponseGoogleClient = TypeVar(
 class _PaginatedResponseBase(TypedDict):
     """Typing info for a paginated response."""
 
-    accessRole: str  # noqa: N815
-    defaultReminders: list[dict[str, object]]  # noqa: N815
+    accessRole: str
+    defaultReminders: list[dict[str, object]]
     etag: str
     kind: Literal["calendar#events"]
-    nextPageToken: str  # noqa: N815
+    nextPageToken: str
     summary: str
-    timeZone: str  # noqa: N815
+    timeZone: str
     updated: str
 
 
@@ -50,9 +51,9 @@ class PaginatedResponseCalendar(_PaginatedResponseBase):
 class PaginatedResponseFit(TypedDict):
     """Paginated response specifically for the Fit client."""
 
-    minStartTimeNs: str  # noqa: N815
-    maxEndTimeNs: str  # noqa: N815
-    dataSourceId: str  # noqa: N815
+    minStartTimeNs: str
+    maxEndTimeNs: str
+    dataSourceId: str
     point: list[dict[str, object]]
 
 
@@ -72,40 +73,23 @@ class GoogleClient(
 ):
     """Custom client for interacting with the Google APIs."""
 
-    ACCESS_TOKEN_ENDPOINT = "https://oauth2.googleapis.com/token"
+    ACCESS_TOKEN_ENDPOINT = "https://oauth2.googleapis.com/token"  # noqa: S105
     AUTH_LINK_BASE = "https://accounts.google.com/o/oauth2/v2/auth"
+    BASE_URL: str
 
-    DEFAULT_PARAMS: dict[StrBytIntFlt, StrBytIntFlt | Iterable[StrBytIntFlt] | None] = {
+    DEFAULT_PARAMS: ClassVar[
+        dict[StrBytIntFlt, StrBytIntFlt | Iterable[StrBytIntFlt] | None]
+    ] = {
         "pageSize": "50",
     }
 
-    def __init__(
-        self,
-        *,
-        client_id: str,
-        client_secret: str,
-        base_url: str,
-        log_requests: bool = False,
-        creds_cache_path: Path | None = None,
-        scopes: list[str] | None = None,
-    ):
-        """Initialise the client."""
-        super().__init__(
-            base_url=base_url,
-            access_token_endpoint=self.ACCESS_TOKEN_ENDPOINT,
-            auth_link_base=self.AUTH_LINK_BASE,
-            client_id=client_id,
-            client_secret=client_secret,
-            log_requests=log_requests,
-            creds_cache_path=creds_cache_path,
-            scopes=scopes,
-        )
-
     def get_items(
         self,
         url: str,
         *,
-        list_key: Literal["albums", "drives", "files", "items", "point"] = "items",
+        list_key: Literal[
+            "albums", "drives", "files", "items", "mediaItems", "point"
+        ] = "items",
         params: dict[
             StrBytIntFlt,
             StrBytIntFlt | Iterable[StrBytIntFlt] | None,
