@@ -1,21 +1,18 @@
-# pylint: disable=too-few-public-methods,no-self-argument
 """Custom client for interacting with Google's Photos API."""
 from __future__ import annotations
 
 from datetime import datetime
-from enum import Enum, auto
+from enum import Enum
 from logging import DEBUG, getLogger
 from pathlib import Path
-from typing import Any, Literal, TypeAlias, TypedDict, TypeVar
+from typing import Any, ClassVar, Literal, Self, TypeAlias
 
-from pydantic import Field, validator
+from pydantic import Field, field_validator
 from requests import post
+from typing_extensions import TypedDict
 
 from wg_utilities.clients._google import GoogleClient
-from wg_utilities.clients.oauth_client import (
-    BaseModelWithConfig,
-    GenericModelWithConfig,
-)
+from wg_utilities.clients.oauth_client import BaseModelWithConfig
 from wg_utilities.functions import force_mkdir
 from wg_utilities.loggers import add_stream_handler
 
@@ -25,41 +22,41 @@ add_stream_handler(LOGGER)
 
 
 class _SharedAlbumOptionsInfo(TypedDict):
-    isCollaborative: bool  # noqa: N815
-    isCommentable: bool  # noqa: N815
+    isCollaborative: bool
+    isCommentable: bool
 
 
 class _ShareInfoInfo(TypedDict):
-    isJoinable: bool  # noqa: N815
-    isJoined: bool  # noqa: N815
-    isOwned: bool  # noqa: N815
-    shareableUrl: str  # noqa: N815
-    sharedAlbumOptions: _SharedAlbumOptionsInfo  # noqa: N815
-    shareToken: str  # noqa: N815
+    isJoinable: bool
+    isJoined: bool
+    isOwned: bool
+    shareableUrl: str
+    sharedAlbumOptions: _SharedAlbumOptionsInfo
+    shareToken: str
 
 
 class _MediaItemMetadataPhoto(BaseModelWithConfig):
-    camera_make: str | None = Field(alias="cameraMake")
-    camera_model: str | None = Field(alias="cameraModel")
-    focal_length: float | None = Field(alias="focalLength")
-    aperture_f_number: float | None = Field(alias="apertureFNumber")
-    iso_equivalent: int | None = Field(alias="isoEquivalent")
-    exposure_time: str | None = Field(alias="exposureTime")
+    camera_make: str | None = Field(None, alias="cameraMake")
+    camera_model: str | None = Field(None, alias="cameraModel")
+    focal_length: float | None = Field(None, alias="focalLength")
+    aperture_f_number: float | None = Field(None, alias="apertureFNumber")
+    iso_equivalent: int | None = Field(None, alias="isoEquivalent")
+    exposure_time: str | None = Field(None, alias="exposureTime")
 
 
 class _MediaItemMetadataVideo(BaseModelWithConfig):
-    camera_make: str | None = Field(alias="cameraMake")
-    camera_model: str | None = Field(alias="cameraModel")
-    status: Literal["READY"] | None
-    fps: float | None
+    camera_make: str | None = Field(None, alias="cameraMake")
+    camera_model: str | None = Field(None, alias="cameraModel")
+    status: Literal["READY"] | None = None
+    fps: float | None = None
 
 
 class _MediaItemMetadata(BaseModelWithConfig):
     creation_time: datetime = Field(alias="creationTime")
     height: int
     width: int
-    photo: _MediaItemMetadataPhoto | None
-    video: _MediaItemMetadataVideo | None
+    photo: _MediaItemMetadataPhoto | None = Field(default=None)
+    video: _MediaItemMetadataVideo | None = Field(default=None)
 
 
 class MediaType(Enum):
@@ -67,28 +64,31 @@ class MediaType(Enum):
 
     IMAGE = "image"
     VIDEO = "video"
-    UNKNOWN = auto()
+    UNKNOWN = "unknown"
 
 
-FJR = TypeVar("FJR", bound="GooglePhotosEntity")
-
-
-class GooglePhotosEntity(GenericModelWithConfig):
-    """Base class for Google Photos entities."""
+class _GooglePhotosEntityJson(TypedDict):
+    """JSON representation of a Google Photos entity."""
 
     id: str
+    productUrl: str
+
+
+class GooglePhotosEntity(BaseModelWithConfig):
+    """Generic base class for Google Photos entities."""
+
+    id: str  # noqa: A003
     product_url: str = Field(alias="productUrl")
 
     google_client: GooglePhotosClient = Field(exclude=True)
 
     @classmethod
     def from_json_response(
-        cls: type[FJR],
+        cls,
         value: GooglePhotosEntityJson,
         *,
         google_client: GooglePhotosClient,
-        _waive_validation: bool = False,
-    ) -> FJR:
+    ) -> Self:
         """Create an entity from a JSON response."""
 
         value_data: dict[str, Any] = {
@@ -96,20 +96,12 @@ class GooglePhotosEntity(GenericModelWithConfig):
             **value,
         }
 
-        instance = cls.parse_obj(value_data)
-
-        if not _waive_validation:
-            instance._validate()  # pylint: disable=protected-access
-
-        return instance
+        return cls.model_validate(value_data)
 
 
-class AlbumJson(TypedDict):
+class AlbumJson(_GooglePhotosEntityJson):
     """JSON representation of an Album."""
 
-    id: str
-    productUrl: str  # noqa: N815
-
     coverPhotoBaseUrl: str  # noqa: N815
     coverPhotoMediaItemId: str  # noqa: N815
     isWriteable: bool | None  # noqa: N815
@@ -123,15 +115,16 @@ class Album(GooglePhotosEntity):
 
     cover_photo_base_url: str = Field(alias="coverPhotoBaseUrl")
     cover_photo_media_item_id: str = Field(alias="coverPhotoMediaItemId")
-    is_writeable: bool | None = Field(alias="isWriteable")
+    is_writeable: bool | None = Field(None, alias="isWriteable")
     media_items_count: int = Field(alias="mediaItemsCount")
-    share_info: _ShareInfoInfo | None = Field(alias="shareInfo")
+    share_info: _ShareInfoInfo | None = Field(None, alias="shareInfo")
     title: str
 
     _media_items: list[MediaItem]
 
-    @validator("title")
-    def _validate_title(cls, value: str) -> str:  # noqa: N805
+    @field_validator("title")
+    @classmethod
+    def _validate_title(cls, value: str) -> str:
         """Validate the title of the album."""
 
         if not value:
@@ -149,18 +142,15 @@ class Album(GooglePhotosEntity):
         """
 
         if not hasattr(self, "_media_items"):
-            self._set_private_attr(
-                "_media_items",
-                [
-                    MediaItem.from_json_response(item, google_client=self.google_client)
-                    for item in self.google_client.get_items(
-                        "/mediaItems:search",
-                        method_override=post,
-                        list_key="mediaItems",
-                        params={"albumId": self.id, "pageSize": 100},
-                    )
-                ],
-            )
+            self._media_items = [
+                MediaItem.from_json_response(item, google_client=self.google_client)
+                for item in self.google_client.get_items(
+                    "/mediaItems:search",
+                    method_override=post,
+                    list_key="mediaItems",
+                    params={"albumId": self.id, "pageSize": 100},
+                )
+            ]
 
         return self._media_items
 
@@ -170,12 +160,9 @@ class Album(GooglePhotosEntity):
         return item.id in [media_item.id for media_item in self.media_items]
 
 
-class MediaItemJson(TypedDict):
+class MediaItemJson(_GooglePhotosEntityJson):
     """JSON representation of a Media Item (photo or video)."""
 
-    id: str
-    productUrl: str  # noqa: N815
-
     baseUrl: str  # noqa: N815
     contributorInfo: dict[str, str] | None  # noqa: N815
     description: str | None
@@ -188,8 +175,10 @@ class MediaItem(GooglePhotosEntity):
     """Class for representing a MediaItem and its metadata/content."""
 
     base_url: str = Field(alias="baseUrl")
-    contributor_info: dict[str, str] | None = Field(alias="contributorInfo")
-    description: str | None
+    contributor_info: dict[str, str] | None = Field(
+        alias="contributorInfo", default=None
+    )
+    description: str | None = Field(default=None)
     filename: str
     media_metadata: _MediaItemMetadata = Field(alias="mediaMetadata")
     mime_type: str = Field(alias="mimeType")
@@ -228,13 +217,10 @@ class MediaItem(GooglePhotosEntity):
                 Path.cwd() if target_directory == "" else Path(target_directory)
             )
 
-        self._set_private_attr(
-            "_local_path",
-            (
-                target_directory
-                / self.creation_datetime.strftime("%Y/%m/%d")
-                / (file_name_override or self.filename)
-            ),
+        self._local_path = (
+            target_directory
+            / self.creation_datetime.strftime("%Y/%m/%d")
+            / (file_name_override or self.filename)
         )
 
         if self.local_path.is_file() and not force_download:
@@ -261,7 +247,7 @@ class MediaItem(GooglePhotosEntity):
         return self.local_path
 
     @property
-    def bytes(self) -> bytes:
+    def binary_content(self) -> bytes:
         """MediaItem binary content.
 
         Opens the local copy of the file (downloading it first if necessary) and
@@ -342,35 +328,17 @@ class GooglePhotosClient(GoogleClient[GooglePhotosEntityJson]):
 
     BASE_URL = "https://photoslibrary.googleapis.com/v1"
 
-    DEFAULT_SCOPES = [
+    DEFAULT_SCOPES: ClassVar[list[str]] = [
         "https://www.googleapis.com/auth/photoslibrary.readonly",
         "https://www.googleapis.com/auth/photoslibrary.appendonly",
         "https://www.googleapis.com/auth/photoslibrary.readonly.appcreateddata",
         "https://www.googleapis.com/auth/photoslibrary.edit.appcreateddata",
     ]
 
-    def __init__(
-        self,
-        client_id: str,
-        client_secret: str,
-        *,
-        scopes: list[str] | None = None,
-        log_requests: bool = False,
-        creds_cache_path: Path | None = None,
-    ):
-        super().__init__(
-            base_url=self.BASE_URL,
-            client_id=client_id,
-            client_secret=client_secret,
-            scopes=scopes or self.DEFAULT_SCOPES,
-            log_requests=log_requests,
-            creds_cache_path=creds_cache_path,
-        )
-
-        self._albums: list[Album]
-        # Only really used to check if all album metadata has been fetched, not
-        # available to the user (would still require caching all albums).
-        self._album_count: int
+    _albums: list[Album]
+    # Only really used to check if all album metadata has been fetched, not
+    # available to the user (would still require caching all albums).
+    _album_count: int
 
     def get_album_by_id(self, album_id: str) -> Album:
         """Get an album by its ID.
@@ -456,5 +424,5 @@ class GooglePhotosClient(GoogleClient[GooglePhotosEntityJson]):
         return self._albums
 
 
-Album.update_forward_refs()
-MediaItem.update_forward_refs()
+Album.model_rebuild()
+MediaItem.model_rebuild()
