@@ -1,5 +1,6 @@
-# type: ignore
-"""Drivers obtained from https://github.com/waveshare/e-Paper"""
+"""Drivers obtained from https://github.com/waveshare/e-Paper/."""
+from __future__ import annotations
+
 from logging import DEBUG, Formatter, StreamHandler, getLogger
 from sys import stdout
 
@@ -27,13 +28,11 @@ else:
     _LOGGER.warning("Unable to import E-Paper Driver, running in test mode")
     FRAME_DELAY = 0
 
-    # pylint: disable=too-few-public-methods
-    class EPD:
-        """Dynamically built class to mirror the functionality of the EPD on devices
-        which don't support it"""
+    class EPD:  # type: ignore[no-redef]
+        """Mirror the functionality of the EPD on devices which don't support it."""
 
     for method in dir(EPD_ORIG):
         if not method.startswith("_"):
-            setattr(EPD, method, lambda *a, **k: None)
+            setattr(EPD, method, lambda *_, **__: None)
 
 __all__ = ["EPD", "EPD_HEIGHT", "EPD_WIDTH", "implementation", "FRAME_DELAY"]
