@@ -1,3 +1,4 @@
+# ruff: noqa
 """Config for the EPD.
 
 * | File        :	  epdconfig.py
@@ -30,7 +31,7 @@ THE SOFTWARE.
 from __future__ import annotations
 
 # pylint: disable=missing-function-docstring,missing-class-docstring
-from logging import debug
+from logging import debug, exception
 from os import path
 from sys import modules
 from time import sleep
@@ -38,7 +39,7 @@ from typing import Literal
 
 
 # noinspection PyMissingOrEmptyDocstring
-class RaspberryPi:  # noqa: D101
+class RaspberryPi:
     # Pin definition
     RST_PIN = 17
     DC_PIN = 25
@@ -48,29 +49,29 @@ class RaspberryPi:  # noqa: D101
     # noinspection PyUnresolvedReferences,PyPackageRequirements
     # pylint: disable=import-outside-toplevel
     def __init__(self) -> None:
-        from RPi import GPIO
-        from spidev import SpiDev
+        from RPi import GPIO  # type: ignore[import-not-found]
+        from spidev import SpiDev  # type: ignore[import-not-found]
 
         self.gpio = GPIO
 
         # SPI device, bus = 0, device = 0
         self.spi = SpiDev(0, 0)
 
-    def digital_write(self, pin: int, value: bool) -> None:  # noqa: D102
+    def digital_write(self, pin: int, value: bool) -> None:
         self.gpio.output(pin, value)
 
-    def digital_read(self, pin: int) -> bool:  # noqa: D102
+    def digital_read(self, pin: int) -> bool:
         return self.gpio.input(pin)  # type: ignore[no-any-return]
 
     @staticmethod
-    def delay_ms(delay_time: float | int) -> None:  # noqa: D102
+    def delay_ms(delay_time: float | int) -> None:
         sleep(delay_time / 1000.0)
 
-    def spi_writebyte(self, data: list[int]) -> None:  # noqa: D102
+    def spi_writebyte(self, data: list[int]) -> None:
         self.spi.writebytes(data)
 
     # noinspection DuplicatedCode
-    def module_init(self) -> Literal[0]:  # noqa: D102
+    def module_init(self) -> Literal[0]:
         self.gpio.setmode(self.gpio.BCM)
         self.gpio.setwarnings(False)
         self.gpio.setup(self.RST_PIN, self.gpio.OUT)
@@ -81,7 +82,7 @@ class RaspberryPi:  # noqa: D101
         self.spi.mode = 0b00
         return 0
 
-    def module_exit(self) -> None:  # noqa: D102
+    def module_exit(self) -> None:
         debug("spi end")
         self.spi.close()
 
@@ -92,76 +93,6 @@ class RaspberryPi:  # noqa: D101
         self.gpio.cleanup()
 
 
-# This is commented out because I don't have a Jetson Nano and I can't be bothered type
-# hinting it all
-
-# pylint: disable=pointless-string-statement
-"""
-class JetsonNano:
-    # Pin definition
-    RST_PIN = 17
-    DC_PIN = 25
-    CS_PIN = 8
-    BUSY_PIN = 24
-
-    # noinspection PyUnresolvedReferences
-    def __init__(self):
-        from ctypes.cdll import LoadLibrary
-
-        from Jetson import GPIO
-
-        self.spi = None
-
-        for find_dir in [
-            dirname(realpath(__file__)),
-            "/usr/local/lib",
-            "/usr/lib",
-        ]:
-            so_filename = join(find_dir, "sysfs_software_spi.so")
-            if exists(so_filename):
-                self.spi = LoadLibrary(so_filename)
-                break
-
-        if self.spi is None:
-            raise RuntimeError("Cannot find sysfs_software_spi.so")
-
-        self.gpio = GPIO
-
-    def digital_write(self, pin, value):
-        self.gpio.output(pin, value)
-
-    def digital_read(self, _):
-        return self.gpio.input(self.BUSY_PIN)
-
-    @staticmethod
-    def delay_ms(delay_time):
-        sleep(delay_time / 1000.0)
-
-    def spi_writebyte(self, data):
-        self.spi.SYSFS_software_spi_transfer(data[0])
-
-    # noinspection DuplicatedCode
-    def module_init(self):
-        self.gpio.setmode(self.gpio.BCM)
-        self.gpio.setwarnings(False)
-        self.gpio.setup(self.RST_PIN, self.gpio.OUT)
-        self.gpio.setup(self.DC_PIN, self.gpio.OUT)
-        self.gpio.setup(self.CS_PIN, self.gpio.OUT)
-        self.gpio.setup(self.BUSY_PIN, self.gpio.IN)
-        self.spi.SYSFS_software_spi_begin()
-        return 0
-
-    def module_exit(self):
-        debug("spi end")
-        self.spi.SYSFS_software_spi_end()
-
-        debug("close 5V, Module enters 0 power consumption ...")
-        self.gpio.output(self.RST_PIN, 0)
-        self.gpio.output(self.DC_PIN, 0)
-
-        self.gpio.cleanup()
-"""
-
 try:
     if path.exists("/sys/bus/platform/drivers/gpiomem-bcm2835"):
         # pylint: disable=used-before-assignment
@@ -174,14 +105,15 @@ try:
             setattr(modules[__name__], func, getattr(implementation, func))
 
     TEST_MODE = False
-except (RuntimeError, ImportError):
-    # pylint: disable=too-few-public-methods
+except (RuntimeError, ImportError) as exc:
+    exception(exc)
+
     class FakeImplementation:
         """Class to cover functionality of implementations on non-supporting devices."""
 
     for method in dir(RaspberryPi):
         if not method.startswith("_"):
-            setattr(FakeImplementation, method, lambda *a, **k: None)
+            setattr(FakeImplementation, method, lambda *_, **__: None)
 
     implementation = FakeImplementation()
 
