@@ -1,6 +1,8 @@
-"""Yamaha YAS-209 Soundbar"""
+"""Yamaha YAS-209 Soundbar."""
 
 
+from __future__ import annotations
+
 from .yamaha_yas_209 import YamahaYas209
 
 __all__ = ["YamahaYas209"]
